'use strict';

const dashboardVersion = '0.1.8a';

let serverAddress;
let context;
let lastHeardLimit;
let dstarnowLimit;
let logLevel;

let logger;

let noraapi;
let kdkapi;
let currentSelectedFunction = 'home';
let dashboardInfo;

$(document).ready(()=>{
    initialize();
});

function initialize() {
    Utils.injectStringIncludes();

    if(!logLevelConfig || "" === logLevelConfig)
        logLevel = 'info';
    else
        logLevel = logLevelConfig;

    log.setDefaultLevel(logLevel);
    logger = log.getLogger(Utils.getFileName());
    logger.setLevel(logLevel);

    let serverPort = 0;
    if(!serverPortConfig || serverPortConfig <= 0)
        serverPort = 3000;
    else
        serverPort = serverPortConfig;

    if(serverAddressConfig && "" !== serverAddressConfig)
        serverAddress = 'http://' + serverAddressConfig + ':' + serverPort;
    else if(location.protocol === 'file:')
        serverAddress = 'http://localhost:' + serverPort;
    else
        serverAddress = location.host;

    if(!contextConfig || "" === contextConfig)
        context = "/nora";
    else
        context = contextConfig;

    if(!lastHeardLimitConfig)
        lastHeardLimit = 20;
    else if(lastHeardLimitConfig <= 0)
        lastHeardLimit = 1;
    else
        lastHeardLimit = lastHeardLimitConfig;

    if(!dstarnowLimitConfig)
        dstarnowLimit = 15;
    else if(dstarnowLimitConfig <= 0)
        dstarnowLimit = 1;
    else
        dstarnowLimit = dstarnowLimitConfig;

    const navbarText = $('#main_navbar_text');
    if(navbarText.exists()){
        navbarText.html(
            'dashboard v' + dashboardVersion +
            '<br />' + serverAddress + context
        );
    }

    setupNoraAPI(serverAddress, context);

    setupKdkAPI('http://kdkapi.k-dk.net:8080', '/kdkapi');
}

function setupKdkAPI(serverAddress, context) {
    kdkapi = new KdkAPI(
        serverAddress, context, 'error'
    );

    kdkapi.connect(
        () => {
            kdkapi.onUpdateHeardEntryEvent = onJDSTARNowReceiveUpdateHeardEntry;
            kdkapi.onUpdateHeardEntriesEvent = onJDSTARNowReceiveUpdateHeardEntries;
            setTimeout(() =>{
                kdkapi.enableJARLDSTARNow = true;
                kdkapi.requestHeardLog();
            }, 1000);
        },
        () => {
        },
        () => {
            showTopMessage("error", "KdkAPI Connection timeout");
        },
        (err) => {
            showTopMessage("error", "KdkAPI Connection error = " + err);
        }
    );
}

function setupNoraAPI(serverAddress, context) {
    noraapi = new NoraAPI(serverAddress, context, logLevel);

    noraapi.connect(
        () =>{
            noraapi.onNotifyLogEvent = onReceiveNotifyLog;

            setTimeout(() =>{
                noraapi.getDashboardInfo().then(onReceiveDashboardInfo);

                clearTopMessage();

                showTopMessage("success", "Connected to server " + serverAddress + ".");
    
                setTimeout(() =>{
                    clearTopMessage();
                }, 10000);
            }, 1000);

            appendLogConsole("info", 'Client has connected to the server!');
        },
        () => {
            clearTopMessage();

            appendLogConsole('info', 'Client has disconnected, reloading...');

            setTimeout(() =>{
                location.reload();
            }, 5000);
        },
        () => {
            showTopMessage(
                "error",
                'Server ' + serverAddress + ' Connection timeout'
            );

            setTimeout(() =>{
                location.reload();
            }, 30000);
        },
        (err) => {
            showTopMessage(
                "error",
                'Server ' + serverAddress + ' Connection error = ' + err
            );

            setTimeout(() =>{
                location.reload();
            }, 30000);
        }
    );
}

function onReceiveDashboardInfo(info) {

    if(!checkDashboardVersion(info))
        window.alert('This dashboard version is too old, please update.');

    dashboardInfo = info;

    createNavbar(info);
    createContents(info);

    onFunctionChange('home', 'home');
}

function createNavbar(data) {

    const gatewayInfo = data.gatewayInfo;
    const repeaterInfos = data.repeaterInfos;
    const reflectorInfos = data.reflectorInfos;
    const routingInfos = data.routingInfos;

    const gatewayCallsign =
        gatewayInfo.gatewayCallsign
        .substr(0, gatewayInfo.gatewayCallsign.length - 1).trim();
    const title = gatewayCallsign + " Dashboard";

    $('title').text(title);

    $('#main_navbar_brand').text(title);
    const navbarText = $('#main_navbar_text');
    if(navbarText.exists()){
        navbarText.html(
            data.applicationName +
            " v" + data.applicationVersion +
            "@" + data.applicationRunningOS +
            ' / ' +
            navbarText.html()
        );
    }


    $('#main_navbar_item_gateway_link').attr('href', "#" + gatewayInfo.webSocketRoomId);

    const navbarItemReflectors = $('#main_navbar_item_reflectors_ul');
    if(navbarItemReflectors.exists()){
        reflectorInfos.forEach(reflectorInfo => {
            const reflectorItem = $("<li/>",{"class":"nav-item dropdown"});
            reflectorItem.append(
                $("<a/>",{
                    "class":"dropdown-item",
                    href : "#" + reflectorInfo.webSocketRoomId,
                    text : reflectorInfo.reflectorType
                })
            );

            navbarItemReflectors.append(reflectorItem);
        });
    }

    repeaterInfos.forEach(repeaterInfo => {
        const repeaterItem = $("<li/>",{"class":"nav-item dropdown"});
        repeaterItem.append(
            $("<a/>",{
                "class":"dropdown-item dropdown-toggle",
                "data-toggle":"dropdown",
                "aria-haspopup":true,
                "aria-expanded":false,
                href : "#" + repeaterInfo.webSocketRoomId,
                text : repeaterInfo.repeaterCallsign
            })
        );
        
        const modemItems =
            $('<ul/>', {
                "class":"dropdown-menu"
            });
        
        modemItems.append(
            $('<li/>').append(
                $('<a/>', {
                        "class":"dropdown-item",
                        "data-toggle" : "tab",
                        href:"#" + repeaterInfo.webSocketRoomId,
                        text:repeaterInfo.repeaterCallsign
                    })
            )	
        );

        modemItems.append(
            $('<div/>', {"class":"dropdown-divider"})
        );

        repeaterInfo.modemInfos.forEach(modemInfo => {
            modemItems.append(
                $('<li/>').append(
                    $('<a/>', {
                        "class":"dropdown-item",
                        href:"#" + modemInfo.webSocketRoomId,
                        text:modemInfo.modemType + "(" + modemInfo.modemId + ")"
                    })
                )
            );

        });
        
        repeaterItem.append(modemItems);
        
        $('#main_navbar_item_repeaters_ul').append(repeaterItem);
    });

    const navbarItemRoutingServices = $('#main_navbar_item_routingservices_ul');
    if(navbarItemRoutingServices.exists()){
        routingInfos.forEach(routingInfo => {
            const routingItem = $("<li/>",{"class":"nav-item dropdown"});
            routingItem.append(
                $("<a/>",{
                    "class":"dropdown-item",
                    href : "#" + routingInfo.webSocketRoomId,
                    text : routingInfo.routingType
                })
            );

            navbarItemRoutingServices.append(routingItem);
        });
    }

    $('#main_navbar').bootnavbar();
}

function createContents(data) {
    setupHome(data);

    const maintab = $('#maintab');

    if(data.gatewayInfo.webSocketRoomId){
        const gatewayTab =
        $('<div/>',{"id":"maintab_" + data.gatewayInfo.webSocketRoomId,"class":"tab-pane"});
        maintab.append(gatewayTab);

        setupContentsGateway(maintab, data.gatewayInfo, data);
    }

    data.repeaterInfos.forEach(repeaterInfo => {
        if(repeaterInfo.webSocketRoomId){
            const repeatertab =
                $(
                    '<div/>',
                    {"id":"maintab_" + repeaterInfo.webSocketRoomId,"class":"tab-pane"}
                );
            
            maintab.append(repeatertab);

            setupContentsRepeater(repeatertab, repeaterInfo, data);
        }

        repeaterInfo.modemInfos.forEach(modemInfo =>{
            if(modemInfo.webSocketRoomId){
                const modemtab = 
                    $('<div/>',{
                        "id":"maintab_" + modemInfo.webSocketRoomId,
                        "class":"tab-pane"
                    });

                maintab.append(modemtab);

                setupContentsModem(modemtab, modemInfo, repeaterInfo, data);
            }
        });
    });

    data.reflectorInfos.forEach(reflectorInfo => {
        if(reflectorInfo.webSocketRoomId){
            const reflectorTab =
            $('<div/>',{"id":"maintab_" + reflectorInfo.webSocketRoomId,"class":"tab-pane"});

            maintab.append(reflectorTab);

            setupContentsReflector(reflectorTab, reflectorInfo);
        }
    });

    data.routingInfos.forEach(routingInfo => {
        if(routingInfo.webSocketRoomId){
            const routingTab =
            $('<div/>',{"id":"maintab_" + routingInfo.webSocketRoomId,"class":"tab-pane"});

            maintab.append(routingTab);

            setupContentsRoutingService(routingTab, routingInfo);
        }
    });
}

function findGatewayRepeaterInfoByModemWebSocketRoomID(webSocketRoomId) {
    const gatewayInfo = dashboardInfo.gatewayInfo;
    let repeaterInfo, modemInfo;
    repeaterInfo = dashboardInfo.repeaterInfos.find(repeaterInfo => {
        modemInfo = repeaterInfo.modemInfos.find(modemInfo => {
            return modemInfo.webSocketRoomId === webSocketRoomId;
        });
        return modemInfo;
    });

    if(gatewayInfo && repeaterInfo && modemInfo)
        return {'gatewayInfo':gatewayInfo,'repeaterInfo':repeaterInfo,'modemInfo':modemInfo};
    else{
        logger.error("Could not found websocketroomid = " + webSocketRoomId);

        return null;
    }
}

function findRepeaterInfoByRepeaterFunctionID(repeaterFunctionID) {
    const repeaterInfo = dashboardInfo.repeaterInfos.find(repeaterInfo => {
        return repeaterInfo.webSocketRoomId == repeaterFunctionID;
    });

    return repeaterInfo;
}

function updateView() {
    if(currentSelectedFunction === 'home')
        $('#main_navbar_item_home').addClass('active');
    else
        $('#main_navbar_item_home').removeClass('active');

    if(currentSelectedFunction === dashboardInfo.gatewayInfo.webSocketRoomId)
        $('#main_navbar_item_gateway').addClass("active");
    else
        $('#main_navbar_item_gateway').removeClass("active");

    if(dashboardInfo.repeaterInfos.find((repeaterInfo) => {
        const result =
            currentSelectedFunction === repeaterInfo.webSocketRoomId ||
            repeaterInfo.modemInfos.find((modemInfo) => {
                return currentSelectedFunction === modemInfo.webSocketRoomId
            });

        return result;
    }))
        $('#main_navbar_item_repeaters').addClass("active");
    else
        $('#main_navbar_item_repeaters').removeClass("active");	
        
    if(dashboardInfo.reflectorInfos.find((reflectorInfo) => {
        return currentSelectedFunction === reflectorInfo.webSocketRoomId;
    }))
        $('#main_navbar_item_reflectors').addClass("active");
    else
        $('#main_navbar_item_reflectors').removeClass("active");

    if(dashboardInfo.routingInfos.find((routingInfo) => {
        return currentSelectedFunction === routingInfo.webSocketRoomId;
    }))
        $('#main_navbar_item_routingservices').addClass("active");
    else
        $('#main_navbar_item_routingservices').removeClass("active");

    const breadcrumb = $('#breadcrumb');
    breadcrumb.empty();
    breadcrumb.append(
        $('<li/>',{
            "class":"breadcrumb-item"
        }).append($('<a/>',{
            "class" : "text-white font-weight-bold",
            href:"#home",
            text:"HOME >"
        }))
    );
    if(currentSelectedFunction.match(dashboardInfo.gatewayInfo.webSocketRoomId)){
        breadcrumb.append(
            $('<li/>',{
                "class":"breadcrumb-item"
            }).append($('<a/>',{
                "class" : "text-white font-weight-bold",
                href:"#" + dashboardInfo.gatewayInfo.webSocketRoomId,
                text:dashboardInfo.gatewayInfo.gatewayCallsign + " >"
            }))
        );
    }
    dashboardInfo.repeaterInfos.forEach((repeaterInfo) => {
        if(currentSelectedFunction.match(repeaterInfo.webSocketRoomId)){
            breadcrumb.append(
                $('<li/>',{
                    "class":"breadcrumb-item"
                }).append($('<a/>',{
                    "class" : "text-white font-weight-bold",
                    href:"#" + repeaterInfo.webSocketRoomId,
                    text:repeaterInfo.repeaterCallsign + " >"
                }))
            );
        }

        const modem =
            repeaterInfo.modemInfos.find((modemInfo) =>{
                return currentSelectedFunction === modemInfo.webSocketRoomId;
            });
        if(modem) {
            breadcrumb.append(
                $('<li/>',{
                    "class":"breadcrumb-item"
                }).append($('<a/>',{
                    "class" : "text-white font-weight-bold",
                    href:"#" + modem.webSocketRoomId,
                    text:modem.modemType + "(" + modem.modemId + ")" + " >"
                }))
            );
            return false;
        }
    });

    dashboardInfo.reflectorInfos.forEach((reflectorInfo) => {
        if(currentSelectedFunction.match(reflectorInfo.webSocketRoomId)){
            breadcrumb.append(
                $('<li/>',{
                    "class":"breadcrumb-item"
                }).append($('<a/>',{
                    "class" : "text-white font-weight-bold",
                    href:"#" + reflectorInfo.webSocketRoomId,
                    text:reflectorInfo.reflectorType + " >"
                }))
            );
        }
    });

    dashboardInfo.routingInfos.forEach((routingInfo) => {
        if(currentSelectedFunction.match(routingInfo.webSocketRoomId)){
            breadcrumb.append(
                $('<li/>',{
                    "class":"breadcrumb-item"
                }).append($('<a/>',{
                    "class" : "text-white font-weight-bold",
                    href:"#" + routingInfo.webSocketRoomId,
                    text:routingInfo.routingType + " >"
                }))
            );
        }
    });

    $('#maintab div[class*="tab-pane"').each((index, tabPanel) =>{
        if(tabPanel.id === ("maintab_" + currentSelectedFunction))
            $(tabPanel).addClass('active');
        else
            $(tabPanel).removeClass('active');
    });
}

function getContentsTab(fuctionID) {
    const tabID = "#maintab_" + selectorEscape(fuctionID);
    const tab = $(tabID);
    if(tab.exists())
        return tab;
    else
        logger.error("Tab not found = " + tabID + ".");

    return false;
}

function onFunctionChange(newFunc, prevFunc) {
    if(newFunc !== prevFunc)
        noraapi.enableDashboard(prevFunc, false);

    noraapi.enableDashboard(newFunc, true);

    logger.debug("Change room to " + newFunc + '<-' + prevFunc);

    currentSelectedFunction = newFunc;

    updateView();

    if(dashboardInfo.gatewayInfo.webSocketRoomId === newFunc){
        const funcID = dashboardInfo.gatewayInfo.webSocketRoomId;
        
        const gatewayTab =
            getContentsTab(dashboardInfo.gatewayInfo.webSocketRoomId);
        if(gatewayTab){
            noraapi.getStatus(newFunc).then((data) =>{
                updateContentsGateway(gatewayTab, data);
            });
    
            noraapi.getHeardLog(funcID).then((data) => {
                updateContentsGatewayHeardLog(gatewayTab, data);
            });

//            noraapi.getReflectorHosts(funcID).then((data) => {
//                updateContentsGatewayReflectorHosts(gatewayTab, data, false)
//            });
        }
    }
    else{
        noraapi.getStatus(newFunc).then((data) =>{
            $(dashboardInfo.repeaterInfos).filter((i, v, a) =>{
                return v.webSocketRoomId === newFunc;
            }).each((i, v) =>{
                const tab = getContentsTab(v.webSocketRoomId);
                if(tab){updateContentsRepeater(tab, data);}
            });
    
            $(dashboardInfo.reflectorInfos).filter((i, v) =>{
                return v.webSocketRoomId === newFunc;
            }).each((i, v) =>{
                const tab = getContentsTab(v.webSocketRoomId);
                if(tab){updateContentsReflector(tab, data);}
            });

            $(dashboardInfo.routingInfos).filter((i, v) =>{
                return v.webSocketRoomId === newFunc;
            }).each((i, v) =>{
                const tab = getContentsTab(v.webSocketRoomId);
                if(tab){updateContentsRoutingService(tab, data);}
            });
        });
    }

}

function checkDashboardVersion(data) {
    const requiredDashboardVersion = data.requiredDashboardVersion;

    const valid =
        requiredDashboardVersion && requiredDashboardVersion <= dashboardVersion;

    logger.trace(
        'Check dashboard version...(Required:' + requiredDashboardVersion +
        '/Current:' + dashboardVersion + ')' +
        (valid ? '[OK]' : '[TOO OLD]')
    );
    
    return valid;
}

$(document).on('click', '.navbar a, .breadcrumb a', function(){
    const selectedFunction = $(this).attr("href").replace(/[^a-z0-9_.]/g, '');
    logger.trace("Navbar clicked = " + selectedFunction);

    if(selectedFunction){
        const roomChanged = currentSelectedFunction !== selectedFunction;
        if(roomChanged)
            onFunctionChange(selectedFunction, currentSelectedFunction);
    }
});
